@@ -4,6 +4,13 @@
 #include <security/pam_appl.h>
 #include <security/pam_client.h>
 
+#ifndef PAM_BP_RCONTROL
+/* Linux-PAM prior to 0.74 */
+#define PAM_BP_RCONTROL	PAM_BP_CONTROL
+#define PAM_BP_WDATA	PAM_BP_DATA
+#define PAM_BP_RDATA	PAM_BP_DATA
+#endif
+
 #include <security/_pam_userpass.h>
 #include <security/pam_userpass.h>
 
@@ -12,24 +19,25 @@ int pam_userpass_conv(int num_msg, const struct pam_message **msg,
 {
 	pam_userpass_t *userpass = (pam_userpass_t *)appdata_ptr;
 	pamc_bp_t prompt;
-	char *data;
+	const char *input;
+	char *output;
 	char flags;
 
 	if (num_msg != 1 || msg[0]->msg_style != PAM_BINARY_PROMPT)
 		return PAM_CONV_ERR;
 
 	prompt = (pamc_bp_t)msg[0]->msg;
-	data = PAM_BP_DATA(prompt);
+	input = PAM_BP_RDATA(prompt);
 
-	if (PAM_BP_CONTROL(prompt) != PAM_BPC_SELECT ||
-	    strncmp(data, USERPASS_AGENT_ID "/", USERPASS_AGENT_ID_LENGTH + 1))
+	if (PAM_BP_RCONTROL(prompt) != PAM_BPC_SELECT ||
+	    strncmp(input, USERPASS_AGENT_ID "/", USERPASS_AGENT_ID_LENGTH + 1))
 		return PAM_CONV_ERR;
 
-	flags = data[USERPASS_AGENT_ID_LENGTH + 1];
-	data += USERPASS_AGENT_ID_LENGTH + 1 + 1;
+	flags = input[USERPASS_AGENT_ID_LENGTH + 1];
+	input += USERPASS_AGENT_ID_LENGTH + 1 + 1;
 
 	if ((flags & USERPASS_USER_MASK) == USERPASS_USER_FIXED &&
-	    strcmp(data, userpass->user))
+	    strcmp(input, userpass->user))
 		return PAM_CONV_AGAIN;
 
 	if (!(*resp = malloc(sizeof(struct pam_response))))
@@ -38,11 +46,11 @@ int pam_userpass_conv(int num_msg, const struct pam_message **msg,
 	prompt = NULL;
 	PAM_BP_RENEW(&prompt, PAM_BPC_DONE,
 		strlen(userpass->user) + 1 + strlen(userpass->pass));
-	data = PAM_BP_DATA(prompt);
+	output = PAM_BP_WDATA(prompt);
 
-	strcpy(data, userpass->user);
-	data += strlen(data) + 1;
-	memcpy(data, userpass->pass, strlen(userpass->pass));
+	strcpy(output, userpass->user);
+	output += strlen(output) + 1;
+	memcpy(output, userpass->pass, strlen(userpass->pass));
 
 	(*resp)[0].resp_retcode = 0;
 	(*resp)[0].resp = (char *)prompt;
